import "Gillothrin.Items.BaseFunc";
import "Gillothrin.Items.GUI";
import "Gillothrin.Items.TooltipWindow";

ItemStat = class( Turbine.UI.Control );
function ItemStat:Constructor(item,name,textWidth,parent)
	Turbine.UI.Control.Constructor(self);
	if(item ~= nil) then				
		self.item = item;

		local quantity = item:GetQuantity();
		local itemStatBackgroundColor = Turbine.UI.Color(rgb(7),rgb(32),rgb(71));
		local numberSlotBackgroundColor = Turbine.UI.Color(rgb(2),rgb(44),rgb(112));
		local charWidthInPixels = 7;
		local labelWidth = 0;
		if(textWidth == -1) then
			labelWidth = string.len(quantity) * charWidthInPixels;
		else
			labelWidth = textWidth * charWidthInPixels;
		end
		
		local backgroundHeight = 18;
		local imgWidth = 15;
		local leftPadding = 1;
		local textStart = imgWidth+3*leftPadding;
		local fgHeight = 16;
		local topPadding = 1;
		
		self:SetSize(labelWidth+leftPadding+textStart,backgroundHeight);
		self:SetBackColor( itemStatBackgroundColor );
		
		local itemText = VLabel(quantity,textStart,topPadding,labelWidth,fgHeight);
		itemText:SetParent(self);
		itemText:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
		itemText:SetBackColor( numberSlotBackgroundColor );
		itemText:SetMouseVisible(false);
		self.itemText = itemText;
		
		local itemImage = CImage(item:GetSmallImage(),leftPadding,topPadding,imgWidth,fgHeight,name);
		itemImage:SetParent(self);
		itemImage:SetMouseVisible(false);
		self.itemImage = itemImage;
		
		local this = self;
		self.ItemQuantityChanged = function(s,a)
			local originalText = this.itemText:GetText();
			local newText = this.item:GetQuantity();
			
			if newText == "0" then debug(this.itemImage.name.." got "..newText); end
			
			if(string.len(originalText) ~= string.len(newText)) then
				_G.eventWalletChange(this,this.item);
			else
				this.itemText:SetText(newText);
			end			
		end
		self.item:Listen(self.ItemQuantityChanged);
		
		self.mouseButton = nil;
	
		self.MouseUpHandler = function(s,a) 
			parent.mouseButton, parent.mousePressedX, parent.mousePressedY = nil,nil,nil;
		end;
		self.MouseDownHandler = function(s,a) 
			parent.mouseButton = a.Button;
			parent.mousePressedX = a.X;
			parent.mousePressedY = a.Y;
		end;
		
		self.MouseMoveHandler = function(s,a) 
			if(parent.mouseButton == Turbine.UI.MouseButton.Left) then
				local left, top = parent:GetPosition();
				local width, height = parent:GetSize();
				local x = left + (a.X - parent.mousePressedX);
				local y = top + (a.Y - parent.mousePressedY);
				local mx,my = ValidatePosition(x,y,width,height);
				parent:SetPosition(mx,my);
				parent.optionsObj.mainLeft = mx;
				parent.optionsObj.mainTop = my;
			end
		end;
		
		AddCallback(self,"MouseUp",self.MouseUpHandler);
		AddCallback(self,"MouseDown",self.MouseDownHandler);
		AddCallback(self,"MouseMove",self.MouseMoveHandler);
		self:SetParent(parent);
		self.tooltip = TooltipWindow(self,self.itemImage.name);
	end
end

function ItemStat:Cleanup()
	self.item:Unhook()
	self:SetParent(nil);
	RemoveCallback(self,"MouseUp",self.MouseUpHandler);
	RemoveCallback(self,"MouseDown",self.MouseDownHandler);
	RemoveCallback(self,"MouseMove",self.MouseMoveHandler);
end