import "Turbine.UI";
import "Turbine.UI.Lotro";
-----------------------------------
import "Gillothrin.Items.BaseFunc";
import "Gillothrin.Items.ItemStat";
import "Gillothrin.Items.GUI";


ItemsWindow = class(Turbine.UI.Window);
function ItemsWindow:Constructor(optionsObj,playerWallet)
	Turbine.UI.Window.Constructor(self);	
	self.itemStats = {};
	self.padding = 10;
	self.defaultRowHeight = 19;
	self.defaultWidth = 300;
	
	self.optionsObj = optionsObj;
	self.maxScreenWidth = Turbine.UI.Display:GetWidth();
	self.playerWallet = playerWallet;
	
	self.isOneline = self.optionsObj.isOneline;
	
	self.defaultHeight = self.defaultRowHeight + self.padding*2;
	if(optionsObj.mainWidth ~= nil) then
		self.mainWidth = optionsObj.mainWidth;
	else
		self.mainWidth = self.defaultWidth;
	end;
	
	if(optionsObj.mainHeight ~= nil) then
		self.mainHeight = optionsObj.mainHeight;
	else 
		self.mainHeight = self.defaultHeight;
	end;
	self:SetSize(self.mainWidth, self.mainHeight);
	self:SetVisible(true);
	
	if debugMode then self:SetBackColor( Turbine.UI.Color(rgb(2),rgb(44),rgb(112) )); end;
	
	if(optionsObj.mainLeft ~= nil) and (optionsObj.mainTop ~= nil) then	
		self:SetPosition(optionsObj.mainLeft,optionsObj.mainTop);
	else
		self:SetDefaultPosition();
	end
	
	local x,y = self:GetWidth(),self:GetHeight();
	local this = self;
	self.rightHorizontalController = WindowController(self,x-self.padding,self.padding,self.padding,y-2*self.padding,function(s,a)
		if(this.mouseButton == Turbine.UI.MouseButton.Left) then
			local left = this:GetLeft();
			local top = this:GetTop();
			local newx = this:GetWidth()+a.X;
			local newy = this:GetHeight();
			this:RePosition(left,top,newx,newy);
			this:RepaintItemInfo();
		end
	end);
	
	self.PositionChangedEventHandler = function(s, a)		
		s.optionsObj.mainLeft = s:GetLeft();
		s.optionsObj.mainTop = s:GetTop();
	end
	AddCallback(self,"PositionChanged",self.PositionChangedEventHandler);
	
	self.SizeChangedEventHandler = function(s,a)		
		s.optionsObj.mainWidth = s:GetWidth();
		s.optionsObj.mainHeight = s:GetHeight();
	end
	AddCallback(self,"SizeChanged",self.SizeChangedEventHandler);
	
	self.mouseButton = nil;
	
	self.MouseUpHandler = function(s,a) 
		s.mouseButton, s.mousePressedX, s.mousePressedY = nil,nil,nil;
	end;
	self.MouseDownHandler = function(s,a) 
		s.mouseButton = a.Button;
		s.mousePressedX = a.X;
		s.mousePressedY = a.Y;
	end;
	self.MouseMoveHandler = function(s,a) 
		if(s.mouseButton == Turbine.UI.MouseButton.Left) then
			local left, top = s:GetPosition();
            local width, height = s:GetSize();
            local x = left + (a.X - s.mousePressedX);
            local y = top + (a.Y - s.mousePressedY);
			local mx,my = ValidatePosition(x,y,width,height);
			s:SetPosition(mx,my);
			s.optionsObj.mainLeft = mx;
			s.optionsObj.mainTop = my;
		end
	end;
	AddCallback(self,"MouseUp",self.MouseUpHandler);
	AddCallback(self,"MouseDown",self.MouseDownHandler);
	AddCallback(self,"MouseMove",self.MouseMoveHandler);
end

function ItemsWindow:Cleanup()
	local v = self.itemStats;
	for i=1,#v do
		v[i]:Cleanup();
	end
	self:GetControls():Clear();
	self.itemStats = {};
end

function ItemsWindow:Unload()
	self:Cleanup();
	self.rightHorizontalController:Cleanup();
	RemoveCallback(self,"PositionChanged",self.PositionChangedEventHandler);
	RemoveCallback(self,"SizeChanged",self.SizeChangedEventHandler);
	RemoveCallback(self,"MouseUp",self.MouseUpHandler);
	RemoveCallback(self,"MouseDown",self.MouseDownHandler);
	RemoveCallback(self,"MouseMove",self.MouseMoveHandler);
end

function ItemsWindow:RepaintItemInfo()
	self:Cleanup();
	self.playerWallet:LoadWallet();
	
	local xCoord = self.padding;
	local phialIdx = 1;
	local maxWidth = 0;
	local actualWidth = 0;
	local foundItems = self.playerWallet:GetFoundItems();
	local pkeys = {};
	for k,v in pairs(foundItems) do 
		if(not self.optionsObj.notSelectedItems[k]) then
			local strlen = string.len(v:GetQuantity());
			actualWidth = actualWidth + strlen;
			maxWidth = math.max(strlen,maxWidth);
			table.insert(pkeys,k);
		end
	end
		
	table.sort(pkeys, function(a,b) return tostring(a) < tostring(b) end);
	 -- some magic stuff since the window have all those decorations
	local row = 0;
	local shouldWrap = false;
	local textWidth = -1; -- -1 for indicating to the ItemStat to set its own size
	local itemStatElementSize = (maxWidth * 7 + 4 + 15);
	local itemStatElementActualSize = (actualWidth * 7 + (4 + 15)*#pkeys);
	local dP = self.padding*2;
	local maxPaintWidth = math.min(self:GetWidth()-dP,itemStatElementActualSize);
	if(itemStatElementActualSize > maxPaintWidth) then
		textWidth = maxWidth;
		shouldWrap = true;
	end
	
	local minItemWidth = 0;
	
	if shouldWrap then minItemWidth = itemStatElementSize; end;
	
	local element = 1;
	for _,k in ipairs(pkeys) do	
		if(xCoord > maxPaintWidth) then			
			row = row + 1;
			xCoord = self.padding;
		end
		
		local p = ItemStat(foundItems[k],k,textWidth,self);		
		p:SetPosition(xCoord, self.padding+self.defaultRowHeight * row);
		self.itemStats[phialIdx] = p;
		phialIdx = phialIdx + 1;
		
		if(shouldWrap) then
			xCoord = xCoord + itemStatElementSize;
		else 
			local miw = p:GetWidth();
			xCoord = xCoord + miw;
			minItemWidth = math.max(miw,minItemWidth);
		end
	end
	local wx = xCoord+self.padding;
	if(shouldWrap)then
		wx = maxPaintWidth+dP;
	end
	local wy = dP+self.defaultRowHeight+(self.defaultRowHeight*row);
	self:SetSize(wx,wy);
	self:SetMinimumWidth(minItemWidth+dP);
	self.rightHorizontalController:RePosition(self,wx-self.padding,self.padding,self.padding,wy-dP);
end

function ItemsWindow:LoadWindow()
	self:RepaintItemInfo();
	self:SetVisible(true);
end

function ItemsWindow:ResetWindow()
	self:SetDefaultPosition();
	self:LoadWindow();
end

function ItemsWindow:SetDefaultPosition()
	self:SetPosition(100,100);
end

function ItemsWindow:RePosition(left,top,x,y)
	self:SetPosition(left,top);
	self:SetSize(x,y);
end

WindowController = class(Turbine.UI.Control);
function WindowController:Constructor(target,left,top,xsize,ysize,moveHandler)
	Turbine.UI.Control.Constructor(self);
	if debugMode then 
		self:SetBackColor( Turbine.UI.Color( math.random(), math.random(), math.random() ) ); 
	end;
	
	self:RePosition(target,left,top,xsize,ysize);
	
	self.MouseUpHandler = function(s,a) 
		target.mouseButton = nil; 
		if not debugMode then
			self:SetBackColor( Turbine.UI.Color( 0, 0, 0, 0 ) );
		end;
	end;
	self.MouseDownHandler = function(s,a) 
		target.mouseButton = a.Button; 
		if not debugMode then
			self:SetBackColor( Turbine.UI.Color( 0.4, 0, 0, 0 ) );
		end;
	end;
	self.MouseEnterHandler = function(s,a) 
		if not debugMode then
			self:SetBackColor( Turbine.UI.Color( 0.4, 0, 0, 0 ) );
		end;
	end;
	self.MouseLeaveHandler = function(s,a)
		if not debugMode then
			if target.mouseButton == nil then
				self:SetBackColor( Turbine.UI.Color( 0, 0, 0, 0 ) );
			end
		end;
	end;
	self.MouseMoveHandler = moveHandler;
	
	AddCallback(self,"MouseUp",self.MouseUpHandler);
	AddCallback(self,"MouseDown",self.MouseDownHandler);
	AddCallback(self,"MouseMove",self.MouseMoveHandler);
	AddCallback(self,"MouseEnter",self.MouseEnterHandler);
	AddCallback(self,"MouseLeave",self.MouseLeaveHandler);
end

function WindowController:RePosition(target,left,top,xsize,ysize)
	self:SetParent(target);
	self:SetPosition(left,top);
	self:SetSize(xsize,ysize);
end

function WindowController:Cleanup()
	RemoveCallback(self,"MouseUp",self.MouseUpHandler);
	RemoveCallback(self,"MouseDown",self.MouseDownHandler);
	RemoveCallback(self,"MouseMove",self.MouseMoveHandler);
	RemoveCallback(self,"MouseEnter",self.MouseEnterHandler);
	RemoveCallback(self,"MouseLeave",self.MouseLeaveHandler);
end
