import "Turbine";
import "Turbine.Gameplay";
import "Gillothrin.Items.Wallet";
import "Gillothrin.Items.ItemsWindow";
import "Gillothrin.Items.DataOptions";
import "Gillothrin.Items.OptionsControl";

ItemsPlugin = class(Turbine.Object);
function ItemsPlugin:Constructor()
	Turbine.Object.Constructor(self);
	
	self.optionsObj = DataOptions();

	self.playerWallet=Wallet(Turbine.Gameplay.LocalPlayer:GetInstance():GetWallet(),self.optionsObj);

	self.itemsWindow = ItemsWindow(self.optionsObj,self.playerWallet);
	self.optionsControl = OptionsControl(self.optionsObj,self.playerWallet);
	
	self.itemsWindow:LoadWindow();
	
	AddCallback(self.playerWallet:GetLOTROWallet(), "ItemAdded", eventWalletChange);
	AddCallback(self.playerWallet:GetLOTROWallet(), "ItemRemoved", eventWalletChange);
	
end

_G.eventWalletChange = function(sender, args)
	itemsPlugin.itemsWindow:RepaintItemInfo();
	itemsPlugin.optionsControl:RepaintOptionsControl();
end

function ItemsPlugin:Unload()
	RemoveCallback(self.playerWallet:GetLOTROWallet(), "ItemAdded", eventWalletChange);
	RemoveCallback(self.playerWallet:GetLOTROWallet(), "ItemRemoved", eventWalletChange);
	_G.eventWalletChange = nil;
	if Turbine.Shell.IsCommand(shellCommandName) then
		Turbine.Shell.RemoveCommand(shellCommandName);
	end

	self.optionsObj:SaveData();
	self.optionsControl:Unload();
	self.itemsWindow:Unload();
	
end

function ItemsPlugin:GetOptionsPanel()
	self.optionsControl:RepaintOptionsControl();
	return self.optionsControl;
end

function ItemsPlugin:RegisterShellCommand()
	myCommandLine = Turbine.ShellCommand();
	local this = self;
	function myCommandLine:Execute( name, shellCommand )
		shellCommand = string.lower(shellCommand);
		if (shellCommand == "setup") or (shellCommand == "options") then
			this.itemsWindow:SetVisible();
			this.optionsControl:ShowOptionsControl();
		elseif (shellCommand == "show") or (shellCommand == "on") then
			this.itemsWindow:SetVisible(true);
			this.itemsWindow:Activate();
		elseif (shellCommand == "hide") or (shellCommand == "off") then
			this.itemsWindow:Close();
		elseif shellCommand == "refresh" then
			eventWalletChange();
		elseif shellCommand == "reset" then
			this.itemsWindow:ResetWindow();
		else
 			myCommandLine:GetHelp();
		end
	end
		
	function myCommandLine:GetShortHelp()
		echo("Command: <rgb=#FFFFFF>/"..shellCommandName.."</rgb> [show|on] [hide|off] [setup|options] [refresh] [reset] [help]");
	end
		
	function myCommandLine:GetHelp()
		echo(string.rep("=",string.len(pluginFullIdText)));
		echo(pluginFullIdText);
		echo(string.rep("=",string.len(pluginFullIdText)));
		echo("USAGE: /"..shellCommandName.." [command]\n");
		echo("COMMANDS:");
		echo("help - show this help (also if no command)");
		echo("show|on - show item tracking window");
		echo("hide|off - hide item tracking window");
		echo("setup|options - show the options window");
		echo("refresh - refresh your tracked items");
		echo("reset - reset the window to default");
		echo("");
	end
	return myCommandLine;
end